@@ -4,8 +4,8 @@ import random
 
 import textwrap
 
-from hypothesis import given
 import hypothesis.strategies as st
+from hypothesis import given
 
 import pytest
 
@@ -208,11 +208,15 @@ class StorageTests(object):
 
         assert 'collection argument must not be given' in str(excinfo.value)
 
-    def test_collection_arg(self, requires_collections, get_storage_args):
-        s = self.storage_class(**get_storage_args(collection='test2'))
-        # Can't do stronger assertion because of radicale, which needs a
-        # fileextension to guess the collection type.
-        assert 'test2' in s.collection
+    def test_collection_arg(self, get_storage_args):
+        if self.supports_collections:
+            s = self.storage_class(**get_storage_args(collection='test2'))
+            # Can't do stronger assertion because of radicale, which needs a
+            # fileextension to guess the collection type.
+            assert 'test2' in s.collection
+        else:
+            with pytest.raises(ValueError):
+                self.storage_class(collection='ayy', **get_storage_args())
 
     def test_case_sensitive_uids(self, s, get_item):
         if s.storage_name == 'filesystem':
