@@ -5,7 +5,7 @@ import functools
 
 from .. import exceptions, sync
 from ..utils import uniq
-from ..utils.compat import with_metaclass
+from ..utils.compat import to_native, to_unicode, with_metaclass
 from ..utils.vobject import Item  # noqa
 
 
@@ -61,9 +61,6 @@ class Storage(with_metaclass(StorageMeta)):
     # The machine-readable name of this collection.
     collection = None
 
-    # The human-readable name of this collection.
-    collection_human = None
-
     # A value of True means the storage does not support write-methods such as
     # upload, update and delete.  A value of False means the storage does
     # support those methods.
@@ -72,8 +69,7 @@ class Storage(with_metaclass(StorageMeta)):
     # The attribute values to show in the representation of the storage.
     _repr_attributes = ()
 
-    def __init__(self, instance_name=None, read_only=None, collection=None,
-                 collection_human=None):
+    def __init__(self, instance_name=None, read_only=None, collection=None):
         if read_only is None:
             read_only = self.read_only
         if self.read_only and not read_only:
@@ -81,10 +77,11 @@ class Storage(with_metaclass(StorageMeta)):
         self.read_only = bool(read_only)
 
         if collection and instance_name:
-            instance_name = '{}/{}'.format(instance_name, collection)
+            # XXX: PY2 hack
+            instance_name = '{}/{}'.format(instance_name,
+                                           to_native(collection, 'utf-8'))
         self.instance_name = instance_name
         self.collection = collection
-        self.collection_human = collection_human
 
     @classmethod
     def discover(cls, **kwargs):
@@ -117,7 +114,13 @@ class Storage(with_metaclass(StorageMeta)):
         raise NotImplementedError()
 
     def __repr__(self):
-        return self.instance_name or '<{}(**{})>'.format(
+        try:
+            if self.instance_name:
+                return str(self.instance_name)
+        except ValueError:
+            pass
+
+        return '<{}(**{})>'.format(
             self.__class__.__name__,
             dict((x, getattr(self, x)) for x in self._repr_attributes)
         )
@@ -213,3 +216,29 @@ class Storage(with_metaclass(StorageMeta)):
         when.
         '''
         yield
+
+    def get_meta(self, key):
+        '''Get metadata value for collection/storage.
+
+        See the vdir specification for the keys that *have* to be accepted.
+
+        :param key: The metadata key.
+        :type key: unicode
+        '''
+
+        raise NotImplementedError('This storage does not support metadata.')
+
+    def set_meta(self, key, value):
+        '''Get metadata value for collection/storage.
+
+        :param key: The metadata key.
+        :type key: unicode
+        :param value: The value.
+        :type value: unicode
+        '''
+
+        raise NotImplementedError('This storage does not support metadata.')
+
+
+def normalize_meta_value(value):
+    return to_unicode(value or u'').strip()
