@@ -79,7 +79,8 @@ def _parse_xml(content):
 
 def _merge_xml(items):
     rv = items[0]
-    rv.extend(items[1:])
+    for item in items[1:]:
+        rv.extend(item.getiterator())
     return rv
 
 
@@ -126,10 +127,10 @@ class Discover(object):
     def find_dav(self):
         try:
             response = self.session.request(
-                'GET', self._well_known_uri, allow_redirects=False,
+                'GET', self._well_known_uri, allow_redirects=True,
                 headers=self.session.get_default_headers()
             )
-            return response.headers.get('Location', '')
+            return response.url
         except (HTTPError, exceptions.Error):
             # The user might not have well-known URLs set up and instead points
             # vdirsyncer directly to the DAV server.
@@ -144,7 +145,7 @@ class Discover(object):
                 return self.find_principal(self.find_dav())
 
         headers = self.session.get_default_headers()
-        headers['Depth'] = 0
+        headers['Depth'] = '0'
         body = """
         <d:propfind xmlns:d="DAV:">
             <d:prop>
@@ -164,7 +165,7 @@ class Discover(object):
         if url is None:
             url = self.find_principal()
         headers = self.session.get_default_headers()
-        headers['Depth'] = 0
+        headers['Depth'] = '0'
         response = self.session.request('PROPFIND', url,
                                         headers=headers,
                                         data=self._homeset_xml)
@@ -180,7 +181,7 @@ class Discover(object):
         if url is None:
             url = self.find_home()
         headers = self.session.get_default_headers()
-        headers['Depth'] = 1
+        headers['Depth'] = '1'
         r = self.session.request('PROPFIND', url, headers=headers,
                                  data=self._collection_xml)
         root = _parse_xml(r.content)
@@ -465,16 +466,24 @@ class DavStorage(Storage):
             data=item.raw.encode('utf-8'),
             headers=headers
         )
+
+        # The server may not return an etag under certain conditions:
+        #
+        #   An origin server MUST NOT send a validator header field (Section
+        #   7.2), such as an ETag or Last-Modified field, in a successful
+        #   response to PUT unless the request's representation data was saved
+        #   without any transformation applied to the body (i.e., the
+        #   resource's new representation data is identical to the
+        #   representation data received in the PUT request) and the validator
+        #   field value reflects the new representation.
+        #
+        # -- https://tools.ietf.org/html/rfc7231#section-4.3.4
+        #
+        # In such cases we return None as etag. The next synchronization will
+        # then detect an etag change (None != some string) and will download
+        # the new item.
         etag = response.headers.get('etag', None)
         href = self._normalize_href(response.url)
-        if not etag:
-            # The server violated the RFC and didn't send an etag. This is
-            # technically a race-condition, but too many popular servers do it.
-            #
-            # ownCloud: https://github.com/owncloud/contacts/issues/920
-            dav_logger.debug('Server did not send etag, fetching {!r}'
-                             .format(href))
-            item2, etag = self.get(href)
         return href, etag
 
     def update(self, href, item, etag):
@@ -536,6 +545,7 @@ class DavStorage(Storage):
             if not etag:
                 dav_logger.warning('Skipping {!r}, etag property is missing.'
                                    .format(href))
+                continue
 
             contenttype = getattr(props.find('{DAV:}getcontenttype'),
                                   'text', None)
@@ -550,7 +560,7 @@ class DavStorage(Storage):
 
     def list(self):
         headers = self.session.get_default_headers()
-        headers['Depth'] = 1
+        headers['Depth'] = '1'
 
         data = '''<?xml version="1.0" encoding="utf-8" ?>
             <D:propfind xmlns:D="DAV:">
@@ -590,7 +600,7 @@ class DavStorage(Storage):
         )
 
         headers = self.session.get_default_headers()
-        headers['Depth'] = 0
+        headers['Depth'] = '0'
 
         response = self.session.request(
             'PROPFIND', '',
@@ -777,7 +787,7 @@ class CaldavStorage(DavStorage):
         # The default in CalDAV's calendar-queries is 0, but the examples use
         # an explicit value of 1 for querying items. it is extremely unclear in
         # the spec which values from WebDAV are actually allowed.
-        headers['Depth'] = 1
+        headers['Depth'] = '1'
 
         for caldavfilter in caldavfilters:
             xml = data.format(caldavfilter=caldavfilter)
