@@ -1,19 +1,18 @@
 # -*- coding: utf-8 -*-
 
 import errno
+import logging
 import os
 import subprocess
-import uuid
 
 from atomicwrites import atomic_write
 
-from .base import Item, Storage
-from .. import exceptions, log
-from ..utils import checkdir, expand_path, get_etag_from_file, \
-    get_etag_from_fileobject
-from ..utils.compat import text_type
+from .base import Item, Storage, normalize_meta_value
+from .. import exceptions
+from ..utils import checkdir, expand_path, generate_href, get_etag_from_file
+from ..utils.compat import text_type, to_native
 
-logger = log.get(__name__)
+logger = logging.getLogger(__name__)
 
 
 class FilesystemStorage(Storage):
@@ -31,7 +30,7 @@ class FilesystemStorage(Storage):
         href, so if you change the file extension after a sync, this will
         trigger a re-download of everything (but *should* not cause data-loss
         of any kind).
-    :param encoding: File encoding for items.
+    :param encoding: File encoding for items, both content and filename.
     :param post_hook: A command to call for each item creation and
         modification. The command will be called with the path of the
         new/updated file.
@@ -43,7 +42,7 @@ class FilesystemStorage(Storage):
     def __init__(self, path, fileext, encoding='utf-8', post_hook=None,
                  **kwargs):
         super(FilesystemStorage, self).__init__(**kwargs)
-        path = expand_path(path)
+        path = expand_path(to_native(path, encoding))
         checkdir(path, create=False)
         self.path = path
         self.encoding = encoding
@@ -71,23 +70,24 @@ class FilesystemStorage(Storage):
     @classmethod
     def create_collection(cls, collection, **kwargs):
         kwargs = dict(kwargs)
+        encoding = kwargs.get('encoding', 'utf-8')
+        path = to_native(kwargs['path'], encoding)
 
         if collection is not None:
-            kwargs['path'] = os.path.join(kwargs['path'], collection)
-        checkdir(expand_path(kwargs['path']), create=True)
+            collection = to_native(collection, encoding)
+            path = os.path.join(path, collection)
+
+        checkdir(expand_path(path), create=True)
+
+        kwargs['path'] = path
         kwargs['collection'] = collection
         return kwargs
 
     def _get_filepath(self, href):
-        return os.path.join(self.path, href)
+        return os.path.join(self.path, to_native(href, self.encoding))
 
-    def _deterministic_href(self, item):
-        # XXX: POSIX only defines / and \0 as invalid chars, but we should make
-        # this work crossplatform.
-        return item.ident.replace('/', '_') + self.fileext
-
-    def _random_href(self):
-        return str(uuid.uuid4()) + self.fileext
+    def _get_href(self, ident):
+        return generate_href(ident) + self.fileext
 
     def list(self):
         for fname in os.listdir(self.path):
@@ -112,7 +112,7 @@ class FilesystemStorage(Storage):
             raise TypeError('item.raw must be a unicode string.')
 
         try:
-            href = self._deterministic_href(item)
+            href = self._get_href(item.ident)
             fpath, etag = self._upload_impl(item, href)
         except OSError as e:
             if e.errno in (
@@ -121,7 +121,8 @@ class FilesystemStorage(Storage):
             ):
                 logger.debug('UID as filename rejected, trying with random '
                              'one.')
-                href = self._random_href()
+                # random href instead of UID-based
+                href = self._get_href(None)
                 fpath, etag = self._upload_impl(item, href)
             else:
                 raise
@@ -135,10 +136,10 @@ class FilesystemStorage(Storage):
         try:
             with atomic_write(fpath, mode='wb', overwrite=False) as f:
                 f.write(item.raw.encode(self.encoding))
-                return fpath, get_etag_from_fileobject(f)
+                return fpath, get_etag_from_file(f)
         except OSError as e:
             if e.errno == errno.EEXIST:
-                raise exceptions.AlreadyExistingError(item)
+                raise exceptions.AlreadyExistingError(existing_href=href)
             else:
                 raise
 
@@ -155,7 +156,7 @@ class FilesystemStorage(Storage):
 
         with atomic_write(fpath, mode='wb', overwrite=True) as f:
             f.write(item.raw.encode(self.encoding))
-            etag = get_etag_from_fileobject(f)
+            etag = get_etag_from_file(f)
 
         if self.post_hook:
             self._run_post_hook(fpath)
@@ -177,3 +178,21 @@ class FilesystemStorage(Storage):
             subprocess.call([self.post_hook, fpath])
         except OSError as e:
             logger.warning('Error executing external hook: {}'.format(str(e)))
+
+    def get_meta(self, key):
+        fpath = os.path.join(self.path, key)
+        try:
+            with open(fpath, 'rb') as f:
+                return normalize_meta_value(f.read().decode(self.encoding))
+        except IOError as e:
+            if e.errno == errno.ENOENT:
+                return u''
+            else:
+                raise
+
+    def set_meta(self, key, value):
+        value = normalize_meta_value(value)
+
+        fpath = os.path.join(self.path, key)
+        with atomic_write(fpath, mode='wb', overwrite=True) as f:
+            f.write(value.encode(self.encoding))
